/**************************************************************************
*@***h* AMSU-A Hydrological FCDR/rama_wfcdr.c
*
* NAME
*   rama_wfcdr.c
*
* PURPOSE
*   Subroutine to read data, generate products and write outputs
*
* DESCRIPTION
*   Program reads AMSU-A 1B* data, generates products
*   and creates AMSU-A NETCDF4 swath file
*
* Input Files 
*   amafile
*
* Output Files
*   None 
*
* Subroutine Called 
*   gnrt_fname.c, openfs.c, rama_hd.c, 
*   rama_scn.c, scn_data.c, init.c, mask_stype.c, 
*   gnrt_stime.c, gnrt_fcdr.c set_sw.c, read_apdata.c, 
*   read_geodata.c, read_asymdata.c, read_itdata.c
*
* Called by
*   AMA2NC_FCDR.c
*
* AUTHOR
*   Huan Meng,  NOAA/NESDIS/STAR/SCSB
*   Wenze Yang, UMD/ESSIC/CICS
*
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE 
*   FINISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL 
*   SUPPORT TO USERS.
*
* HISTORY
*   6/25/2015 Initial Version prepared for NCEI
*
*@*****
**************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"

/*****************************************************************/
void gnrt_fname();
FILE *openfs();
void rama_hd();
void rama_scn();
void scn_data();
void init();
char** mask_stype();
void gnrt_stime();
void gnrt_fcdr();
void set_sw();
void read_apdata();
int  read_geodata();
void read_asymdata();
void read_itdata();

/*****************************************************************/
void rama_wfcdr(FILE *fp_FNAME, 
              char ama_direct[], 
              char geo_direct[],
              char out_direct[])
{

   FILE        *fp_AMA1B;

   short int   numscan, i;
   short int   inx_orb = 0;
   short int   hs, he;
   int	       hrint, ret;

//   char        ama_direct[]="../ama2nc_input_data/"; 
//   char	 geo_direct[]="../geo_input_data/";
   char        nc_name[120], bin_name[120], filename[80], ch[80], fname[80];
   char	       hr[3];
   char        oldsid[]="   ";

/*-----------------------------------------------------*
 * Open AMSU-A 1B orbital files 
 *-----------------------------------------------------*/
   while(fgets(fname,FN_length_AMA,fp_FNAME) != NULL)
   {
     fgets(ch,80,fp_FNAME);   /* get rid of the rest of the space */
//     printf("\nrama_wfcdr/AMA file name: %s.\n",fname);

     strcpy(hdf_fname, fname);
     strcpy(filename, ama_direct);
     strcat(filename, fname);   

     fp_AMA1B = openfs(filename);

     rama_hd(fp_AMA1B);

/*-----------------------------------------------------*
 * If an orbit has scan number larger than an allowed
 * limit (MAXSCANLINE_A = 950), it'll be cut to the
 * limit to prevent the program from crushing.
 *
 * If an orbit has scan number smaller than an allowed
 * limit (MINSCANLINE_A = 100), it'll be excluded
 * to prevent the program from crushing.
 *-----------------------------------------------------*/
     numscan = hblock.last_scan_record;
     printf("Scanline number in this orbit: %d\n", numscan);

     if( numscan > MAXSCANLINE_A)     
     {
         numscan = MAXSCANLINE_A;
         printf("Warning: The number of scanline is larger than the \n");
         printf("maximum scanline number allowed. The program is running\n");
         printf("by setting numscan = MAXSCANLINE_A, but you might need\n");
         printf("to change MAXSCANLINE_A to a larger number to use the \n");
         printf("real numscan. \n");
     }     

     if( numscan < MINSCANLINE_A)     
     {
         printf("Too few scanlines, no process %s.\n",fname); 
         fclose(fp_AMA1B);
         continue;
     }

     rama_scn(fp_AMA1B, numscan);
  
     scn_data(numscan);

     fclose(fp_AMA1B);

     if (hblock.sat_id != 12 && hblock.sat_id != 6) // not MetOp-A, NOAA-17
     {
        ret = read_geodata(geo_direct, fname, numscan);
        if (ret == 1) continue;
     }

/*--------------------------------------------------------*
 * Generate scan times
 *--------------------------------------------------------*/
     gnrt_stime(numscan);
//     printf ("Finish generating the scan times \n");

/*-----------------------------------------------------*
 * Generate NETCDF4 swath file name
 *-----------------------------------------------------*/
     //strncpy(nc_name, gnrt_fname(), HDF_FNL);
     gnrt_fname(nc_name,out_direct,fname,numscan);
     printf("NC4 file name %s, sid is %s\n",nc_name, sid);
     if (strcmp(oldsid, sid) != 0)
     {
//        printf("Need read APC and ASYM data.\n");
        sprintf(oldsid, sid);
        read_apdata(sid);
//        read_asymdata("NaN");
        read_asymdata(sid);
        read_itdata();
//        read_itdata(sid);
     }
//     else
//        printf("No need to read APC or ASYM data.\n");
     

/*-----------------------------------------------------*
 * Initialize the AT and product arrays
 *-----------------------------------------------------*/

     init();
 
/*-----------------------------------------------------*
 * Determine surface type 
 *-----------------------------------------------------*/
     stype_a1_1 = mask_stype(numscan, lat_a1_1, lon_a1_1);
     stype_a1_2 = mask_stype(numscan, lat_a1_2, lon_a1_2);
     stype_a2 = mask_stype(numscan, lat_a2, lon_a2);

/*-----------------------------------------------------*
 * Calculate ATs from counts and generate products 
 *-----------------------------------------------------*/

     gnrt_fcdr(numscan);
//     printf("Finish generating all products \n");
 
/*-----------------------------------------------------*
 * Define and write to the swath file
 *-----------------------------------------------------*/
//     printf("rama_wfcdr_filename: %s\n", nc_name);
     set_sw(nc_name, numscan);

//     printf("rama_wfcdr/Finish one swath NETCDF4 file !\n");
     printf("*********************************************\n\n");

     inx_orb += 1;
   }

   printf("\n-------------------------------------------------------------\n");
   printf("rama_wfcdr/    TOTAL OF %d GRANULES ARE PROCESSED!\n",inx_orb);
   printf("-------------------------------------------------------------\n");

}  /* end of rama_wfcdr.c */
